import { createToken, createPlugin, unescape } from 'fusion-core';
import { UniversalEventsToken } from 'fusion-plugin-universal-events';
import { FetchToken } from 'fusion-tokens';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
const I18nLoaderToken = createToken('I18nLoaderToken');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var createLoader = null;

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env node */
// exported for testing

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env browser */
function loadTranslations() {
  const element = document.getElementById('__TRANSLATIONS__');

  if (!element) {
    throw new Error('[fusion-plugin-i18n] - Could not find a __TRANSLATIONS__ element');
  }

  try {
    return JSON.parse(unescape(element.textContent));
  } catch (e) {
    throw new Error('[fusion-plugin-i18n] - Error parsing __TRANSLATIONS__ element content');
  }
}

const HydrationStateToken = createToken('HydrationStateToken');

const pluginFactory$1 = () => createPlugin({
  deps: {
    fetch: FetchToken.optional,
    hydrationState: HydrationStateToken.optional,
    events: UniversalEventsToken.optional
  },
  provides: ({
    fetch = window.fetch,
    hydrationState,
    events
  } = {}) => {
    class I18n {
      constructor() {
        const {
          localeCode,
          translations
        } = hydrationState || loadTranslations();
        this.requestedKeys = new Set();
        this.translations = translations || {};

        if (localeCode) {
          this.locale = localeCode;
        }
      }

      load(translationKeys) {
        return new Promise(function ($return, $error) {
          const loadedKeys = Object.keys(this.translations);
          const unloaded = translationKeys.filter(key => loadedKeys.indexOf(key) < 0 && !this.requestedKeys.has(key));

          if (unloaded.length > 0) {
            // Don't try to load translations again if a request is already in
            // flight. This means that we need to add unloaded chunks to
            // loadedChunks optimistically and remove them if some error happens
            unloaded.forEach(key => {
              this.requestedKeys.add(key);
            });
            const fetchOpts = {
              method: 'POST',
              headers: _objectSpread({
                Accept: '*/*',
                'Content-Type': 'application/json'
              }, this.locale ? {
                'X-Fusion-Locale-Code': this.locale
              } : {}),
              body: JSON.stringify(unloaded)
            }; // TODO(#3) don't append prefix if injected fetch also injects prefix

            return $return(fetch(`/_translations${this.locale ? `?localeCode=${this.locale}` : ''}`, fetchOpts).then(r => r.json()).then(data => {
              for (const key in data) {
                this.translations[key] = data[key];
                this.requestedKeys.delete(key);
              }
            }).catch(err => {
              // An error occurred, so remove the chunks we were trying to load
              // from loadedChunks. This allows us to try to load those chunk
              // translations again
              unloaded.forEach(key => {
                this.requestedKeys.delete(key);
              });
              throw err;
            }));
          }

          return $return();
        }.bind(this));
      }

      translate(key, interpolations = {}) {
        const template = this.translations[key];

        if (typeof template !== 'string') {
          events && events.emit('i18n-translate-miss', {
            key
          });
          return key;
        }

        return template.replace(/\${(.*?)}/g, (_, k) => interpolations[k] === void 0 ? '${' + k + '}' : String(interpolations[k]));
      }

    }

    const i18n = new I18n();
    return {
      from: () => i18n
    };
  }
});

var clientPlugin = true && pluginFactory$1();

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
const I18nToken = createToken('I18nToken');
const plugin = clientPlugin;

export default plugin;
export { I18nToken, I18nLoaderToken, HydrationStateToken, createLoader as createI18nLoader };
