import { createToken, createPlugin, unescape } from 'fusion-core';
import { UniversalEventsToken } from 'fusion-plugin-universal-events';
import { FetchToken } from 'fusion-tokens';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var I18nLoaderToken = createToken('I18nLoaderToken');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var createLoader = null;

var _templateObject = /*#__PURE__*/ _taggedTemplateLiteralLoose(["\n            <script type=\"application/json\" id=\"__TRANSLATIONS__\">\n              ", "\n            </script>\n          "], ["\n            <script type=\"application/json\" id=\"__TRANSLATIONS__\">\n              ", "\n            </script>\n          "]);

function _taggedTemplateLiteralLoose(strings, raw) { if (!raw) { raw = strings.slice(0); } strings.raw = raw; return strings; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env node */
// exported for testing

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env browser */
function loadTranslations() {
  var element = document.getElementById('__TRANSLATIONS__');

  if (!element) {
    throw new Error('[fusion-plugin-i18n] - Could not find a __TRANSLATIONS__ element');
  }

  try {
    return JSON.parse(unescape(element.textContent));
  } catch (e) {
    throw new Error('[fusion-plugin-i18n] - Error parsing __TRANSLATIONS__ element content');
  }
}

var HydrationStateToken = createToken('HydrationStateToken');

var pluginFactory$1 = function pluginFactory() {
  return createPlugin({
    deps: {
      fetch: FetchToken.optional,
      hydrationState: HydrationStateToken.optional,
      events: UniversalEventsToken.optional
    },
    provides: function provides(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
          _ref$fetch = _ref.fetch,
          fetch = _ref$fetch === void 0 ? window.fetch : _ref$fetch,
          hydrationState = _ref.hydrationState,
          events = _ref.events;

      var I18n =
      /*#__PURE__*/
      function () {
        function I18n() {
          var _ref2 = hydrationState || loadTranslations(),
              localeCode = _ref2.localeCode,
              translations = _ref2.translations;

          this.requestedKeys = new Set();
          this.translations = translations || {};

          if (localeCode) {
            this.locale = localeCode;
          }
        }

        var _proto = I18n.prototype;

        _proto.load = function load(translationKeys) {
          return new Promise(function ($return, $error) {
            var _this = this;

            var loadedKeys = Object.keys(this.translations);
            var unloaded = translationKeys.filter(function (key) {
              return loadedKeys.indexOf(key) < 0 && !_this.requestedKeys.has(key);
            });

            if (unloaded.length > 0) {
              // Don't try to load translations again if a request is already in
              // flight. This means that we need to add unloaded chunks to
              // loadedChunks optimistically and remove them if some error happens
              unloaded.forEach(function (key) {
                _this.requestedKeys.add(key);
              });
              var fetchOpts = {
                method: 'POST',
                headers: _objectSpread({
                  Accept: '*/*',
                  'Content-Type': 'application/json'
                }, this.locale ? {
                  'X-Fusion-Locale-Code': this.locale
                } : {}),
                body: JSON.stringify(unloaded)
              }; // TODO(#3) don't append prefix if injected fetch also injects prefix

              return $return(fetch("/_translations" + (this.locale ? "?localeCode=" + this.locale : ''), fetchOpts).then(function (r) {
                return r.json();
              }).then(function (data) {
                for (var key in data) {
                  _this.translations[key] = data[key];

                  _this.requestedKeys.delete(key);
                }
              }).catch(function (err) {
                // An error occurred, so remove the chunks we were trying to load
                // from loadedChunks. This allows us to try to load those chunk
                // translations again
                unloaded.forEach(function (key) {
                  _this.requestedKeys.delete(key);
                });
                throw err;
              }));
            }

            return $return();
          }.bind(this));
        };

        _proto.translate = function translate(key, interpolations) {
          if (interpolations === void 0) {
            interpolations = {};
          }

          var template = this.translations[key];

          if (typeof template !== 'string') {
            events && events.emit('i18n-translate-miss', {
              key: key
            });
            return key;
          }

          return template.replace(/\${(.*?)}/g, function (_, k) {
            return interpolations[k] === void 0 ? '${' + k + '}' : String(interpolations[k]);
          });
        };

        return I18n;
      }();

      var i18n = new I18n();
      return {
        from: function from() {
          return i18n;
        }
      };
    }
  });
};

var clientPlugin = true && pluginFactory$1();

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var I18nToken = createToken('I18nToken');
var plugin = clientPlugin;

export default plugin;
export { I18nToken, I18nLoaderToken, HydrationStateToken, createLoader as createI18nLoader };
